import json

import requests

from rcbu.common.constants import IDENTITY_TOKEN_URL


def authenticate(username, apikey=None, password=None):
    assert password or apikey
    if apikey:
        return _auth(username=username, apikey=apikey)

    else:
        return _auth(username=username, password=password)


def get_token(username, apikey=None, password=None):
    r = authenticate(username=username, apikey=apikey, password=password)
    return r['access']['token']['id']


def _auth(username, apikey=None, password=None):
    creds = {'username': username}

    auth_type = None
    if password:
        creds['password'] = password
        auth_type = {'passwordCredentials': creds}
    else:
        creds['apiKey'] = apikey
        auth_type = {'RAX-KSKEY:apiKeyCredentials': creds}

    wrapper = {'auth': auth_type}
    data = json.dumps(wrapper)
    r = requests.post(IDENTITY_TOKEN_URL, data=data,
                      headers={'content-type': 'application/json'})
    if not r.ok:
        r.raise_for_status()
    return r.json()



import rcbu.utils.date as date


def _args_from_dict(body):
    return {
        '_id': body['ID'],
        '_type': body['Type'],
        '_parent_id': body['ParentId'],
        '_name': body['DisplayName'],
        '_deleted': body['IsBackupConfigurationDeleted'],
        '_source': {
            'agent_id': body['SourceMachineAgentId'],
            'name': body['SourceMachineName']
        },
        '_destination': {
            'agent_id': body['DestinationMachineAgentId'],
            'name': body['DestinationMachineName']
        },
        '_state': body['CurrentState'],
        '_time': body['TimeOfActivity']
    }


def from_dict(body):
    args = _args_from_dict(body)
    return Activity(**args)


class Activity(object):
    def __init__(self, **kwargs):
        [setattr(self, k, v) for k, v in kwargs.items()]

    def __repr__(self):
        form = ('<{0}Activity id:{1} name:{2} state:{3} time:{4}>')
        pretty_type = self.type[0].upper() + self.type[1:].lower()
        return form.format(pretty_type, self.id, self.name, self.state,
                           self.time.isoformat())

    @property
    def id(self):
        return self._id

    @property
    def type(self):
        return self._type

    @property
    def parent(self):
        return self._parent_id

    @property
    def name(self):
        return self._name

    @property
    def deleted(self):
        return self._deleted

    @property
    def source(self):
        src = self._source
        return '{0} {1}'.format(src['name'], src['agent_id'])

    @property
    def destination(self):
        if self.type.lower() == 'restore':
            dst = self._destination
            return '{0} {1}'.format(dst['name'], dst['agent_id'])
        return None

    @property
    def state(self):
        return self._state

    @property
    def time(self):
        return date.parse(self._time)





def public_key():
    return (b'-----BEGIN PUBLIC KEY-----\n'
            b'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAz'
            b'/3AuYbBdy83TjnmqE0FmGRBa64HtbGkKS0U4nHmE2yT0B'
            b'DxLywPGUSlf/od9ASJJiul+oxNHqO4sY/NguDD9NOI0FB'
            b'm1Mg73wT5PNmv3EPiwRpvsF2IeF0ZQDDPiQ/66dLzEZSs'
            b'AB6oDomJkE1KFCfdwrgd+LxYYG7+pmH+gxTkUDwSwv91M'
            b'ET8Huo+LIRj3StoDgfvBnv3IKNgSCkmgVtmIcB25tJMVw'
            b'n5bJAbuMkMnoOJMvHcAc53wpNl1lWV1wx4i6+69nnyxi8'
            b'Ba+li/zEVW5pOtkP0EYkKCyaeiwdwZnIea9iFhHXHNWdT'
            b'JWex9kRaD4oHWpcj1Oem3b2/twIDAQAB\n'
            b'-----END PUBLIC KEY-----')



import json
import unittest

from httpretty import HTTPretty, httprettified

from rcbu.common.auth import IDENTITY_TOKEN_URL
from rcbu.client.connection import Connection
import tests.mock.auth as mock_auth
import tests.mock.restore as mock_restore
import rcbu.client.restore as restore


def _mock_auth(status):
    reply = mock_auth.authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


class TestRestore(unittest.TestCase):
    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.connection = Connection('a', 'b')
        self.cmd = restore.Restore(1, connection=self.connection)

    @httprettified
    def test_start_works(self):
        url = '{0}/restore/action-requested'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url)
        self.cmd.start()

    @httprettified
    def test_start_encrypted_works(self):
        url = '{0}/restore/action-requested'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url)
        self.cmd._encrypted = True
        self.cmd._encrypted_password = 'taco'
        self.cmd.start()

    @httprettified
    def test_stop_works(self):
        url = '{0}/restore/action-requested'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url)
        self.cmd.stop()


class TestRestoreLoad(unittest.TestCase):
    def test_restore_from_dict_works(self):
        mock = mock_restore.restore()
        restore_obj = restore.from_dict(mock)
        self.assertEqual(restore_obj.overwrites, False)
        self.assertEqual(restore_obj.id, 1)
        self.assertEqual(restore_obj.state, 'Unknown')



from __future__ import print_function
import os
import shutil
import unittest

import requests

from rcbu.client.connection import Connection
from rcbu.client.client import Client
import rcbu.common.schedule as schedule
import rcbu.client.backup_configuration as backup_config
import tests.mock.configuration as mock_config
from tests.utils.credentials import Credentials
from tests.utils.agent import Agent


class TestBackupConfiguration(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.creds = Credentials()
        cls.connection = Connection(cls.creds.name, apikey=cls.creds.key)
        cls.email = cls.creds.email
        cls.agent = Agent()
        os.mkdir('a')

    def setUp(self):
        self.connection = TestBackupConfiguration.connection
        self.client = Client(self.connection)
        self.email = TestBackupConfiguration.email
        self.agent = TestBackupConfiguration.agent
        config = mock_config.backup_configuration(
            name='integration', email=self.email,
            agent_id=self.agent.id
        )
        self.backup_config = backup_config.from_dict(config, self.connection)
        self.backup_config.include(['a'])
        self.backup_config.create()

    def test_fetching_works(self):
        bconf = self.backup_config
        self.assertEqual(bconf.name, 'integration')
        self.assertEqual(bconf.agent_id, 187801)
        self.assertEqual(bconf.email, 'cpp.cabrera@gmail.com')
        self.assertEqual(bconf.notify_on_success, False)
        self.assertEqual(bconf.notify_on_failure, True)
        self.assertEqual(bconf.encrypted, False)
        self.assertEqual(bconf.enabled, True)
        self.assertEqual(bconf.deleted, False)
        self.assertEqual(bconf.schedule, schedule.manually())
        self.assertEqual(bconf.inclusions, set([os.path.realpath('a')]))
        self.assertEqual(bconf.exclusions, set())

    def test_can_rename(self):
        old_name = self.backup_config.name
        self.backup_config.rename('taco')
        self.assertIsNone(self.backup_config.update())
        self.backup_config.reload()
        self.assertNotEqual(old_name, self.backup_config.name)
        self.assertEqual(self.backup_config.name, 'taco')

    def test_can_disable_then_re_enable(self):
        self.assertEqual(self.backup_config.enabled, True)
        self.backup_config.disable()
        self.assertEqual(self.backup_config.enabled, False)
        self.backup_config.enable()
        self.assertEqual(self.backup_config.enabled, True)

    def _reschedule_test(self, xschedule):
        self.backup_config.reschedule(xschedule)
        self.backup_config.update()
        self.assertEqual(self.backup_config.schedule, xschedule)

    def test_can_reschedule_to_daily(self):
        self._reschedule_test(schedule.daily(hour=1, minute=1))

    def test_can_reschedule_to_hourly(self):
        self._reschedule_test(schedule.hourly(interval=4))

    def test_can_reschedule_to_weekly(self):
        self._reschedule_test(schedule.weekly(0, hour=1, minute=1))

    def test_can_add_inclusions(self):
        os.mkdir('b')
        self.backup_config.include(['b'])
        self.backup_config.update()
        self.assertEqual(self.backup_config.inclusions,
                         {os.path.realpath(p) for p in ['a', 'b']})
        shutil.rmtree('b')

    def test_can_add_exclusions(self):
        os.mkdir('b')
        excl_paths = [os.path.join('b', str(i)) for i in range(10)]
        for p in excl_paths:
            with open(p, 'wb') as f:
                f.write(b'1')
        self.backup_config.include(['b'])
        self.backup_config.exclude(excl_paths)
        self.backup_config.update()
        self.assertEqual(self.backup_config.inclusions,
                         {os.path.realpath(p) for p in ['a', 'b']})
        self.assertEqual(self.backup_config.exclusions,
                         {os.path.realpath(p) for p in excl_paths})
        shutil.rmtree('b')

    def test_delete_works(self):
        config = mock_config.backup_configuration(
            name='deleter', email=self.email,
            agent_id=self.agent.id
        )
        conf = backup_config.from_dict(config, self.connection)
        conf.include(['a'])
        conf.create()
        conf.delete()
        with self.assertRaises(requests.exceptions.HTTPError):
            conf.reload()
        self.assertEqual(conf.deleted, True)

    def tearDown(self):
        self.backup_config.delete()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree('a')



"""Mocking restore objects."""


def restore(restore_id=1, backup_id=1, overwrites=False,
            backup_config_id=1, backup_config_name='mock',
            src_id=1, src_name='mock', dest_id=1, dest_name='mock',
            dest_path='/mock', encrypted=False, password='mock'):
    return {
        'RestoreId': restore_id,
        'BackupId': backup_id,
        'OverwriteFiles': overwrites,
        'BackupConfigurationId': backup_config_id,
        'BackupConfigurationName': backup_config_name,
        'BackupMachineId': src_id,
        'BackupMachineName': src_name,
        'DestinationMachineId': dest_id,
        'DestinationMachineName': dest_name,
        'DestinationPath': dest_path,
        'IsEncrypted': encrypted,
        'EncryptedPassword': password,
        'PublicKey': {
            'ModulusHex': 1,
            'ExponentHex': 1
        }
    }



import enum


_methods = 'head get put post delete patch options'.split()
Http = enum.Enum('Http', _methods)
_etr = {k: v for k, v in zip(Http, _methods)}


def enum_to_method(http_enum_value):
    return _etr[http_enum_value]



import json
import unittest

from httpretty import HTTPretty, httprettified

from rcbu.common.auth import IDENTITY_TOKEN_URL
from rcbu.client.connection import Connection
import tests.mock.auth as mock_auth
import rcbu.client.backup as backup


def _mock_auth(status):
    reply = mock_auth.authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


class TestBackup(unittest.TestCase):
    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.connection = Connection('a', 'b')
        self.cmd = backup.Backup(1, 2, connection=self.connection)

    @httprettified
    def test_start_works_and_sets_id(self):
        url = '{0}/backup/action-requested'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url, body=json.dumps(100))
        self.cmd.start()
        self.assertEqual(self.cmd.id, 100)

    @httprettified
    def test_stop_works(self):
        url = '{0}/backup/action-requested'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url)
        self.cmd.stop()



import sys

from rcbu.client.connection import Connection
from rcbu.client.client import Client
import rcbu.client.backup_configuration as backup_config


if len(sys.argv) != 3:
    print('usage: {0} username password'.format(sys.argv[0]))
    quit()

username = sys.argv[1]
password = sys.argv[2]

print('Connecting...')
conn = Connection(username, password=password)
client = Client(conn)
print('Connected!')

print('Creating a new configuration...')
myconf = backup_config.from_file('backup_config.json')
myconf.connect(conn)
myconf.create()
print('Created!')

print('Starting a backup...')
backup = client.create_backup(myconf)
status = backup.start()
print('Started! (id: {0})'.format(backup.id))

print('Waiting for it to finish...')
backup.wait_for_completion(poll_interval=0.5)
print('Done!')

print('...and the result is...')
report = backup.report
report.raise_if_not_ok()
print('OK!')

print('The backup took: {0}'.format(report._time['duration']))





def backup_configuration(agent_id=1, name='mock',
                         is_active=True, is_deleted=False, is_encrypted=False,
                         retention=0, missed_backup_action=1,
                         frequency='Manually', hour=None, minute=None,
                         am_pm=None, weekday=None, interval=None,
                         timezone='Eastern Standard Time',
                         email='mock@mock.com', on_success=False,
                         on_failure=True, inclusions=[], exclusions=[]):
    return {
        'MachineAgentId': agent_id,
        'BackupConfigurationName': name,
        'IsActive': is_active,
        'IsDeleted': is_deleted,
        'IsEncrypted': is_encrypted,
        'VersionRetention': retention,
        'MissedBackupActionId': missed_backup_action,
        'Frequency': frequency,
        'StartTimeHour': hour,
        'StartTimeMinute': minute,
        'StartTimeAmPm': am_pm,
        'DayOfWeekId': weekday,
        'HourInterval': interval,
        'TimeZoneId': timezone,
        'NotifyRecipients': email,
        'NotifySuccess': on_success,
        'NotifyFailure': on_failure,
        'Inclusions': inclusions,
        'Exclusions': exclusions
    }



import sys

from rcbu.client.connection import Connection
from rcbu.client.client import Client


if len(sys.argv) != 3:
    print('usage: {0} username password'.format(sys.argv[0]))
    quit()

username = sys.argv[1]
password = sys.argv[2]

print('Connecting...')
conn = Connection(username, password=password)
client = Client(conn)
print('Connected!')

print("Grabbing the core maintainer's agent...")
ag = [agent for agent in client.agents if
      agent.name.startswith('agent-test-ubuntu')][0]
print('Minion, do my bidding!')

print('Grabbing the first backup configuration I find...')
conf = ag.backup_configurations[0]
print('This is how you will perform this backup...')
print('Inclusions:')
print('    ', conf.inclusions)
print('Exclusions:')
print('    ', conf.exclusions)

print('Starting a backup...')
backup = client.create_backup(conf)
backup.start()
print('Started! (id: {0})'.format(backup.id))

print('Waiting for it to finish...')
backup.wait_for_completion(poll_interval=0.5)
print('Done!')

print('Checking backup...')
backup_report = backup.report
backup_report.raise_if_not_ok()
print('All good!')

print('Now creating a restore from this successful backup...')
restore = client.create_restore(backup.id, source_agent=ag,
                                destination_path='/root',
                                destination_agent=ag,  # by default
                                overwrite=False)  # by default, too
print('Created restore: {0}'.format(restore.id))

print('Now restoring...')
restore.start()
restore.wait_for_completion(poll_interval=.5)
print('Restore complete!')

print('...and the result is...')
restore_report = restore.report
restore_report.raise_if_not_ok()
print('OK!')

print('The backup took: {0}'.format(backup_report._time['duration']))
print('The restore took: {0}'.format(restore_report._time['duration']))



import json
import unittest

from httpretty import HTTPretty, httprettified

import rcbu.common.activity_mixin as activities
from rcbu.client.connection import Connection
from rcbu.common.constants import IDENTITY_TOKEN_URL
from tests.mock.activity import activity
from tests.mock.auth import authenticate

AGENT_ID = 1


class Mock(activities.ExposesActivities):
    def __init__(self, connection, oid):
        activities.ExposesActivities.__init__(self, connection, oid)


class TestActivityMixin(unittest.TestCase):
    @httprettified
    def setUp(self):
        reply = authenticate()
        HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                               status=200, body=json.dumps(reply))
        self.connection = Connection('a', 'b')
        self.agent = Mock(self.connection, AGENT_ID)
        self.url = '{0}/system/activity/{1}'.format(self.connection.host,
                                                    AGENT_ID)

    @httprettified
    def _activity_test(self, method, xtype, xstatus, xcount):
        activities = [activity(activity_id=i,
                               type_tag=xtype,
                               state=xstatus)
                      for i in range(xcount)]
        HTTPretty.register_uri(HTTPretty.GET, self.url,
                               body=json.dumps(activities))
        result = list(getattr(self.agent, method))
        self.assertEqual(len(result), xcount)
        for a in result:
            self.assertEqual(a.type, xtype)
            self.assertEqual(a.state, xstatus)

    @httprettified
    def _busy_test(self, xbusy, count=5):
        activities = [activity(activity_id=i,
                               type_tag='Restore',
                               state='Completed')
                      for i in range(count)]
        extra_jobs_status = 'Complete' if not xbusy else 'InProgress'
        activities.extend(activity(activity_id=i,
                                   type_tag='Backup',
                                   state=extra_jobs_status)
                          for i in range(count))
        HTTPretty.register_uri(HTTPretty.GET, self.url,
                               body=json.dumps(activities))
        self.assertEqual(self.agent.busy, xbusy)

    def test_backup_history_returns_expected(self):
        self._activity_test('backup_history', 'Backup', 'Completed', 5)

    def test_restore_history_returns_expected(self):
        self._activity_test('restore_history', 'Restore', 'Completed', 5)

    def test_cleanup_history_returns_expected(self):
        self._activity_test('cleanup_history', 'Cleanup', 'Completed', 5)

    def test_active_backups_returns_expected(self):
        self._activity_test('active_backups', 'Backup', 'InProgress', 5)

    def test_active_restores_returns_expected(self):
        self._activity_test('active_restores', 'Restore', 'InProgress', 5)

    def test_active_cleanups_returns_expected(self):
        self._activity_test('active_cleanups', 'Cleanup', 'InProgress', 5)

    def test_active_returns_expected(self):
        self._activity_test('active', 'Cleanup', 'InProgress', 5)

    def test_history_returns_expected(self):
        self._activity_test('history', 'Restore', 'Completed', 5)

    def test_busy_returns_true_when_there_are_running_jobs(self):
        self._busy_test(True)

    def test_busy_returns_false_when_agent_is_idle(self):
        self._busy_test(False)





def agent(id_='1', version='1.065959', vault_size='2 GB',
          data_center='ORD', ip_address='0.0.0.0',
          name='mock', os='linux', os_version='13.04',
          encrypted=False, disabled=False, status='Unknown'):
    return {
        'MachineAgentId': id_,
        'AgentVersion': version,
        'BackupVaultSize': vault_size,
        'CleanupAllowed': True,
        'Datacenter': data_center,
        'IPAddress': ip_address,
        'MachineName': name,
        'OperatingSystem': os,
        'OperatingSystemVersion': os_version,
        'IsEncrypted': encrypted,
        'IsDisabled': disabled,
        'Status': status
    }



import unittest
from functools import partial

import rcbu.common.schedule as schedule
import tests.mock.schedule as mock_schedule


class TestSchedule(unittest.TestCase):
    def _check(self, val, xval):
        checker = (partial(self.assertEqual, second=xval) if xval is not None
                   else self.assertIsNone)
        return checker(val)

    def _expect(self, sched, xfrequency=None, xinterval=None,
                xweekday=None, xhour=None, xperiod=None, xminute=None):
        self._check(sched.frequency, xfrequency)
        self._check(sched.interval, xinterval)
        self._check(sched.weekday, xweekday)
        self._check(sched.hour, xhour)
        self._check(sched.period, xperiod)
        self._check(sched.minute, xminute)

    def test_manual_schedule_works(self):
        sched = schedule.manually()
        self._expect(sched, 'Manually')

    def test_weekly_schedule_works(self):
        sched = schedule.weekly(0, 1, 1)
        self._expect(sched, 'Weekly',
                     xweekday='Sunday', xhour=1, xminute=1,
                     xperiod='AM')

    def test_daily_schedule_works(self):
        sched = schedule.daily(1, 1)
        self._expect(sched, 'Daily',
                     xhour=1, xminute=1, xperiod='AM')

    def test_hourly_schedule_works(self):
        sched = schedule.hourly(2)
        self._expect(sched, 'Hourly', xinterval=2)

    def test_load_manual_schedule_works(self):
        mock = mock_schedule.schedule(schedule.ScheduleFrequency.Manual)
        sched = schedule.from_dict(mock)
        self._expect(sched, 'Manually')

    def test_load_weekly_schedule_works(self):
        mock = mock_schedule.schedule(schedule.ScheduleFrequency.Weekly,
                                      weekday=0, hour=0, minute=15)
        sched = schedule.from_dict(mock)
        self._expect(sched, 'Weekly', xweekday='Sunday',
                     xhour=0, xperiod='AM', xminute=15)

    def test_load_daily_schedule_works(self):
        mock = mock_schedule.schedule(schedule.ScheduleFrequency.Daily,
                                      hour=13, minute=15)
        sched = schedule.from_dict(mock)
        self._expect(sched, 'Daily',
                     xhour=1, xperiod='PM', xminute=15)

    def test_load_hourly_schedule_works(self):
        mock = mock_schedule.schedule(schedule.ScheduleFrequency.Hourly,
                                      interval=4)
        sched = schedule.from_dict(mock)
        self._expect(sched, 'Hourly',
                     xinterval=4)

    def test_load_unknown_schedule_raises(self):
        mock = mock_schedule.schedule(8)
        with self.assertRaises(ValueError):
            schedule.from_dict(mock)

    def test_manual_repr_matches_expected(self):
        sched = schedule.manually()
        self.assertEqual(repr(sched),
                         '<Schedule frequency:Manually weekday:* time:*>')

    def test_weekly_repr_matches_expected(self):
        sched = schedule.weekly(0, 1, 1)
        self.assertEqual(repr(sched),
                         ('<Schedule frequency:Weekly weekday:Sunday '
                          'time:01:01 AM>'))

    def test_daily_repr_matches_expected(self):
        sched = schedule.daily(13, 15)
        self.assertEqual(repr(sched),
                         ('<Schedule frequency:Daily weekday:* '
                          'time:01:15 PM>'))

    def test_hourly_repr_matches_expected(self):
        sched = schedule.hourly(12)
        self.assertEqual(repr(sched),
                         ('<Schedule frequency:Hourly weekday:* '
                          'time:* every 12 hours>'))



import json
import os
import unittest

from httpretty import HTTPretty, httprettified

from rcbu.client.connection import Connection
from rcbu.common.constants import IDENTITY_TOKEN_URL
import rcbu.common.schedule as schedule
import tests.mock.auth as mock_auth
import tests.mock.configuration as mock_config
import rcbu.client.backup_configuration as backup_config
from rcbu.common.exceptions import (
    DisconnectedError, InconsistentInclusionsError
)


def _mock_auth(status):
    reply = mock_auth.authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


def _mock_config(status, endpoint, config_id):
    reply = mock_config.backup_configuration(config_id)
    url = '{0}/{1}/{2}'.format(endpoint, 'backup-configuration', config_id)
    HTTPretty.register_uri(HTTPretty.GET, url, json.dumps(reply))


def _quick_config(connection=None):
    return backup_config.from_dict(mock_config.backup_configuration(),
                                   connection=connection)


class TestLoadBackupConfiguration(unittest.TestCase):
    def test_from_dict_works_as_expected(self):
        body = mock_config.backup_configuration()
        config = backup_config.from_dict(body)
        self.assertEqual(config.id, 0)
        self.assertEqual(config.agent_id, 1)
        self.assertEqual(config.notify_on_failure, True)
        self.assertEqual(config.notify_on_success, False)
        self.assertEqual(config.email,
                         'mock@mock.com')
        self.assertEqual(config.name, 'mock')
        self.assertEqual(config.encrypted, False)
        self.assertEqual(config.enabled, True)
        self.assertEqual(config.deleted, False)

    def test_from_dict_throws_key_error_when_missing_attr(self):
        data_keys = mock_config.backup_configuration().keys()
        for key in data_keys:
            mock = mock_config.backup_configuration()
            del mock[key]
            with self.assertRaises(KeyError):
                backup_config.from_dict(mock)


class TestBackupConfiguration(unittest.TestCase):
    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.connection = Connection(username='a', password='a')
        config_api = mock_config.backup_configuration()
        self.config = backup_config.from_dict(config_api, self.connection)

    def test_id_matches_expected(self):
        self.assertEqual(self.config.id, 0)

    def test_agent_id_matches_expected(self):
        self.assertEqual(self.config.agent_id, 1)

    def test_name_matches_expected(self):
        self.assertEqual(self.config.name, 'mock')

    def test_notification_settings_match_expected(self):
        self.assertEqual(self.config.email, 'mock@mock.com')
        self.assertEqual(self.config.notify_on_success, False)
        self.assertEqual(self.config.notify_on_failure, True)

    def test_enabled_matches_expected(self):
        self.assertEqual(self.config.enabled, True)

    def test_encrypted_matches_expected(self):
        self.assertEqual(self.config.encrypted, False)

    def test_deleted_matches_expected(self):
        self.assertEqual(self.config.deleted, False)

    def test_schedule_matches_expected(self):
        self.assertEqual(self.config.schedule, schedule.manually())

    def test_reschedule_works(self):
        self.config.reschedule(schedule.hourly(13))
        self.assertEqual(self.config.schedule, schedule.hourly(13))

    def test_update_notification_settings_works(self):
        self.config.update_notification_settings(email='woot',
                                                 notify_on_failure=False,
                                                 notify_on_success=True)
        self.assertEqual(self.config.email, 'woot')
        self.assertEqual(self.config.notify_on_success, True)
        self.assertEqual(self.config.notify_on_failure, False)

    def test_rename_works(self):
        self.config.rename('woot')
        self.assertEqual(self.config.name, 'woot')

    def test_disconneted_error_raised_if_no_connection(self):
        self.config.connect(None)
        with self.assertRaises(DisconnectedError):
            self.config.enable()

    @httprettified
    def _test_toggle(self, enabled=True):
        url = '{0}/{1}/{2}/{3}'.format(self.connection.host,
                                       'backup-configuration', 'enable',
                                       self.config.id)
        HTTPretty.register_uri(HTTPretty.POST, url, status=200,
                               body=json.dumps({'IsActive': enabled}))
        if enabled:
            self.config.enable()
        else:
            self.config.disable()
        self.assertEqual(self.config.enabled, enabled)

    def test_disable_works(self):
        self._test_toggle(enabled=False)

    def test_enable_works(self):
        self._test_toggle(enabled=True)

    @httprettified
    def test_delete_works(self):
        url = '{0}/backup-configuration/{1}'.format(self.connection.host,
                                                    self.config.id)
        HTTPretty.register_uri(HTTPretty.DELETE, url, status=204)
        self.assertEqual(self.config.deleted, False)
        self.config.delete()
        self.assertEqual(self.config.deleted, True)

    def test_excluding_nonexistent_path_raises_error(self):
        with self.assertRaises(IOError):
            self.config.exclude('not_found')

    def test_including_nonexistent_path_raises_error(self):
        with self.assertRaises(IOError):
            self.config.exclude('not_found')

    def test_including_duplicates_strips_dups(self):
        name = 'setup.py'
        self.config.include([name, name])
        self.assertEqual(self.config.inclusions, {os.path.realpath(name)})

    def test_excluding_duplicates_strips_dups(self):
        name = 'setup.py'
        self.config.exclude([name, name])
        self.assertEqual(self.config.exclusions, {os.path.realpath(name)})

    def test_excluding_included_files_raises_inconsistent(self):
        name = 'setup.py'
        self.config.include([name])
        with self.assertRaises(InconsistentInclusionsError):
            self.config.exclude([name])

    def test_including_excluded_files_raises_inconsistent(self):
        name = 'setup.py'
        self.config.exclude([name])
        with self.assertRaises(InconsistentInclusionsError):
            self.config.include([name])

    @httprettified
    def test_reload_works(self):
        url = '{0}/backup-configuration/{1}'.format(self.connection.host,
                                                    self.config.id)
        conf = mock_config.backup_configuration(agent_id=100)
        HTTPretty.register_uri(HTTPretty.GET, url, status=200,
                               body=json.dumps(conf))
        self.config.reload()
        self.assertEqual(self.config.agent_id, 100)

    @httprettified
    def test_update_works(self):
        url = '{0}/backup-configuration/{1}'.format(self.connection.host,
                                                    self.config.id)
        HTTPretty.register_uri(HTTPretty.PUT, url, status=200)
        self.assertIsNone(self.config.update())

    @httprettified
    def test_create_works(self):
        url = '{0}/backup-configuration'.format(self.connection.host)
        HTTPretty.register_uri(HTTPretty.POST, url, status=200,
                               body=json.dumps({'BackupConfigurationId': 100}))
        old_id = self.config.id
        self.config.create()
        self.assertEqual(self.config.id, 100)
        self.assertNotEqual(self.config.id, old_id)



from rcbu.common.schedule import ScheduleFrequency


def schedule(freq,
             interval=None, weekday=None,
             hour=None, minute=None, period=None):
    return {
        'Frequency': ScheduleFrequency.to_api(freq),
        'StartTimeMinute': minute,
        'StartTimeHour': hour,
        'StartTimeAmPm': period,
        'HourInterval': interval,
        'DayOfWeekId': weekday
    }



class DisconnectedError(RuntimeError):
    def __init__(self):
        msg = 'Must self.connect before continuing.'
        RuntimeError.__init__(self, msg)


class OperationFailed(RuntimeError):
    def __init__(self, op, report):
        message = '{0}: {1}:{2}'.format(op, report._diagnostics, report.errors)
        RuntimeError.__init__(self, message)


class BackupFailed(OperationFailed):
    def __init__(self, report):
        OperationFailed.__init__(self, 'Backup', report)


class RestoreFailed(OperationFailed):
    def __init__(self, report):
        OperationFailed.__init__(self, 'Restore', report)


class InconsistentInclusionsError(ValueError):
    def __init__(self, diff_set):
        msg_template = '{0} are included and excluded. Which did you mean?'
        ValueError.__init__(self, msg_template.format(diff_set))


class NoEndpointFound(RuntimeError):
    def __init__(self, username, region):
        msg = 'No endpoint found for user {0} in region {1}'
        super(NoEndpointFound, self).__init__(msg.format(username, region))



import json

from rcbu.client.command import Command
from rcbu.common.http import Http


def _args_from_dict(body):
    return {
        '_backup_id': body['BackupId'],
        '_overwrites': body['OverwriteFiles'],
        '_backup_config_id': body['BackupConfigurationId'],
        '_backup_config_name': body['BackupConfigurationName'],
        '_source_machine_id': body['BackupMachineId'],
        '_source_machine_name': body['BackupMachineName'],
        '_destination_agent_id': body['DestinationMachineId'],
        '_destination_machine_name': body['DestinationMachineName'],
        '_destination_path': body['DestinationPath'],
        '_encrypted': body['IsEncrypted'],
        '_encrypted_password': body['EncryptedPassword'],
        '_key': {
            'modulus': body['PublicKey']['ModulusHex'],
            'exponent': body['PublicKey']['ExponentHex']
        }
    }


def from_dict(body, connection=None):
    args = _args_from_dict(body)
    return Restore(body['RestoreId'], connection=connection, **args)


class Restore(Command):
    def __init__(self, restore_id, connection, **kwargs):
        Command.__init__(self, restore_id, 'restore', connection, **kwargs)

    def __repr__(self):
        form = ('<Restore id:{0} state:{1} running:{2}>')
        return form.format(self.id, self.state, self.running)

    def _action(self, starting):
        url = '{0}/restore/action-requested'.format(self._connection.host)
        action = 'StartManual' if starting else 'StopManual'
        data = {'Action': action, 'Id': self.id}
        if getattr(self, '_encrypted', None):
            data['EncryptedPassword'] = self._encrypted_password
        resp = self._connection.request(Http.post, url,
                                        data=json.dumps(data))
        self._state = 'Preparing' if starting else 'Stopped'
        return resp

    @property
    def overwrites(self):
        return self._overwrites

    def start(self):
        return self._action(starting=True)

    def stop(self):
        return self._action(starting=False)



import datetime
import unittest

import rcbu.client.backup_report as backup_report
import tests.mock.report as mock_report
from rcbu.common.exceptions import BackupFailed


class TestBackupReport(unittest.TestCase):
    def setUp(self):
        mock = mock_report.backup_report(errors=['explosions'],
                                         restorable=True)
        self.report = backup_report.from_dict(1, mock)

    def test_repr_matches_expected(self):
        form = ('<BackupReport id:{0} state:{1} ok:{2} started:{3} '
                'duration:{4} #errors:{5} bytes:{6}>')
        form = form.format(self.report.id, self.report.state, self.report.ok,
                           self.report.started.isoformat(), '0:00:00',
                           len(self.report.errors),
                           self.report.bytes_stored)
        self.assertEqual(repr(self.report), form)

    def test_id_matches_expected(self):
        self.assertEqual(self.report.id, 1)

    def test_state_matches_expected(self):
        self.assertEqual(self.report.state, 'Completed')

    def test_errors_match_expected(self):
        self.assertEqual(len(self.report.errors), 1)
        self.assertEqual(self.report.errors[0], 'explosions')

    def test_outcome_matches_expected(self):
        self.assertEqual(self.report.outcome, 'OK')

    def test_ok_matches_expected(self):
        self.assertEqual(self.report.ok, True)

    def test_diagnostics_matches_expected(self):
        self.assertEqual(self.report.diagnostics, 'OK')

    def test_started_matches_expected(self):
        expect = datetime.datetime.fromtimestamp(1351118760.000)
        self.assertEqual(self.report.started, expect)

    def test_ended_matches_expected(self):
        expect = datetime.datetime.fromtimestamp(1351118760.001)
        self.assertEqual(self.report.ended, expect)

    def test_duration_matches_expected(self):
        self.assertEqual(self.report.duration, 0)

    def test_does_not_raise_if_ok(self):
        self.report.raise_if_not_ok()

    def test_raises_if_not_restorable(self):
        self.report._restorable = False
        with self.assertRaises(BackupFailed):
            self.report.raise_if_not_ok()

    def test_searched_matches_expected(self):
        self.assertEqual(self.report.files_searched, 0)
        self.assertEqual(self.report.bytes_searched, 2 * 2**30)

    def test_stored_matches_expected(self):
        self.assertEqual(self.report.files_stored, 0)
        self.assertEqual(self.report.bytes_stored, 2 * 2**30)



import json
import unittest

from httpretty import HTTPretty, httprettified, Response

from rcbu.common.auth import IDENTITY_TOKEN_URL
from rcbu.client.connection import Connection
from rcbu.utils.perf import Timer
import tests.mock.auth as mock_auth
import tests.mock.report as mock_report
import rcbu.client.command as command


def _mock_auth(status):
    reply = mock_auth.authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


def _mock_status(status_id, status):
    return {
        'RestoreStateId': status_id,
        'CurrentState': status
    }


def _mock_status_done():
    return _mock_status(3, 'Completed')


def _mock_status_working():
    return _mock_status(2, 'InProgress')


class TestCommand(unittest.TestCase):
    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.connection = Connection('a', 'b')
        self.cmd = command.Command(1, 'backup', self.connection)

    def test_default_state_matches_expected(self):
        self.assertEqual(self.cmd.state, 'Unknown')

    def test_id_matches_expected(self):
        self.assertEqual(self.cmd.id, 1)

    def test_connect_works(self):
        self.cmd.connect(None)
        self.assertEqual(self.cmd._connection, None)

    def test_running_matches_expected(self):
        self.assertEqual(self.cmd.running, False)

    @httprettified
    def test_get_report_works(self):
        url = '{0}/{1}/report/{2}'.format(self.connection.host, self.cmd._type,
                                          self.cmd.id)
        mock = mock_report.backup_report()
        HTTPretty.register_uri(HTTPretty.GET, url, body=json.dumps(mock))
        report = self.cmd.report
        self.assertEqual(report.id, 1)
        self.assertTrue(report.ok)

    @httprettified
    def test_wait_for_completion_throws_on_timeout(self):
        url = '{0}/{1}/{2}'.format(self.connection.host, self.cmd._type,
                                   self.cmd.id)
        HTTPretty.register_uri(HTTPretty.GET, url,
                               body=json.dumps(_mock_status_working()))
        with self.assertRaises(RuntimeError):
            self.cmd.wait_for_completion(poll_interval=.005, timeout=.0025)

    @httprettified
    def test_wait_for_completion_works_poll_none(self):
        url = '{0}/{1}/{2}'.format(self.connection.host, self.cmd._type,
                                   self.cmd.id)
        HTTPretty.register_uri(HTTPretty.GET, url,
                               body=json.dumps(_mock_status_done()))
        with Timer() as timed:
            self.cmd.wait_for_completion(poll_interval=.005)
        self.assertLess(timed.elapsed, .005)

    @httprettified
    def test_wait_for_completion_works_poll_once(self):
        url = '{0}/{1}/{2}'.format(self.connection.host, self.cmd._type,
                                   self.cmd.id)
        resps = [
            Response(body=json.dumps(_mock_status_working())),
            Response(body=json.dumps(_mock_status_done()))
        ]
        HTTPretty.register_uri(HTTPretty.GET, url, responses=resps)
        with Timer() as timed:
            self.cmd.wait_for_completion(poll_interval=.005)
        self.assertGreater(timed.elapsed, .005)
        self.assertLess(timed.elapsed, .015)



def assert_is_none(name, value):
    """A convenience function for reporting on values that should be None.
    Raises a ValueError."""
    if value is not None:
        template = '{0} should be None, not {1}'
        raise ValueError(template.format(name, value))


def assert_bounded(name, lower, upper, value):
    """A convenience function for handling and reporting errors
    on bounded arguments. Raises a ValueError."""
    if value < lower or value > upper:
        template = '{0} {1} out of bounds [{2} - {3}]'
        raise ValueError(template.format(name, value, lower, upper))



"""Used for parsing agent metadata on the local machine."""
import base64
import json
import os


def _path():
    expander = os.path.expandvars
    joiner = os.path.join
    base = ('/etc/driveclient' if os.name != 'nt' else
            joiner(expander('%programdata%'), 'driveclient'))
    return joiner(base, 'bootstrap.json')


def _get_conf():
    path = _path()
    agent_conf = None
    with open(path, 'rt') as f:
        agent_conf = json.load(f)
    return agent_conf


class Agent(object):
    def __init__(self):
        conf = _get_conf()
        self._id = conf['AgentId']
        self._tenant = conf['AccountId']
        self._user = conf['Username']
        self._key = base64.b64decode(conf['AgentKey'])
        self._registered = conf['IsRegistered']

    @property
    def id(self):
        return self._id

    @property
    def tenant(self):
        return self._tenant

    @property
    def user(self):
        return self._user

    @property
    def key(self):
        return self._key

    @property
    def registered(self):
        return self._registered



from rcbu.common.assertions import assert_bounded


def seconds(time):
    '''Given %H:%M:%S -> seconds. Hours can be arbitrarily large.'''
    try:
        hours, minutes, seconds = [int(f) for f in time.split(':')]
    except ValueError:
        msg = 'expecting format %H:%M:%S, not {0}'.format(time)
        raise ValueError(msg)

    assert_bounded('minutes', 0, 59, minutes)
    assert_bounded('seconds', 0, 59, seconds)
    return hours * 3600 + minutes * 60 + seconds


def tuple(seconds):
    '''Returns (hours, minutes, seconds) from seconds.'''
    return (seconds // 3600,
            (seconds // 60) - ((seconds // 3600) * 60),
            seconds % 60)



import os
import json

import rcbu.common.schedule as schedule
from rcbu.common.exceptions import (
    InconsistentInclusionsError, DisconnectedError
)
from rcbu.common.http import Http


def _parse_paths(paths):
    return {p['FilePath'] for p in paths}


def _path_type(path):
    return 'Folder' if os.path.isdir(path) else 'File'


def _paths_to_json(paths):
    return [{'FilePath': p,
             'FileItemType': _path_type(p)}
            for p in paths]


def _args_from_dict(resp):
    """Returns a {} appropriate for constructing a BackupConfiguration."""
    args = {
        '_agent_id': resp['MachineAgentId'],
        '_name': resp['BackupConfigurationName'],
        '_enabled': resp['IsActive'],
        '_deleted': resp['IsDeleted'],
        '_encrypted': resp['IsEncrypted'],
        '_retention': resp['VersionRetention'],
        '_on_missed_backup': resp['MissedBackupActionId'],
        '_time_zone': resp['TimeZoneId'],
        '_schedule': schedule.from_dict(resp),
        '_notify': {
            'email': resp['NotifyRecipients'],
            'on_success': resp['NotifySuccess'],
            'on_failure': resp['NotifyFailure']
        },
        '_inclusions': _parse_paths(resp['Inclusions']),
        '_exclusions': _parse_paths(resp['Exclusions'])
    }
    return args


def _raise_if_overlapping(lhs, rhs):
    diff = lhs & rhs
    if len(diff) > 0:
        raise InconsistentInclusionsError(diff)


def _raise_if_not_exists(path):
    if not os.path.exists(path):
        raise IOError(path)


def from_dict(resp, connection=None):
    args = _args_from_dict(resp)
    return BackupConfiguration(resp.get('BackupConfigurationId', 0),
                               connection,
                               **args)


def from_file(path, connection=None):
    data = None
    with open(path, 'rt') as f:
        data = json.load(f)
    return from_dict(data, connection)


def to_json(config):
    resp = {
        'MachineAgentId': config.agent_id,
        'BackupConfigurationName': config.name,
        'IsActive': config.enabled,
        'VersionRetention': config._retention,
        'MissedBackupActionId': config._on_missed_backup,
        'TimeZoneId': config._time_zone,
        'NotifyRecipients': config.email,
        'NotifySuccess':  config.notify_on_success,
        'NotifyFailure': config.notify_on_failure,
        'Inclusions': _paths_to_json(config._inclusions),
        'Exclusions': _paths_to_json(config._exclusions)
    }
    resp.update(config.schedule.to_api())
    return json.dumps(resp)


class BackupConfiguration(object):
    def __init__(self, config_id, connection=None, **kwargs):
        self._config_id = config_id
        self._inclusions = set()
        self._exclusions = set()
        [setattr(self, k, v) for k, v in kwargs.items()]
        self._connection = connection

    def __repr__(self):
        form = ('<BackupConfiguration id:{0} name:{1} agent:{2} '
                'encrypted:{3} enabled:{4} '
                '#inclusions:{5} #exclusions:{6}>')
        return form.format(self.id, self.name, self.agent_id,
                           self.encrypted, self.enabled,
                           len(self.inclusions),
                           len(self.exclusions))

    @property
    def id(self):
        return getattr(self, "_config_id", None)

    @property
    def agent_id(self):
        return self._agent_id

    @property
    def email(self):
        return self._notify['email']

    @property
    def notify_on_success(self):
        return self._notify['on_success']

    @property
    def notify_on_failure(self):
        return self._notify['on_failure']

    def update_notification_settings(self, email, notify_on_failure=True,
                                     notify_on_success=False):
        self._notify['email'] = email
        self._notify['on_failure'] = notify_on_failure
        self._notify['on_success'] = notify_on_success

    @property
    def name(self):
        return self._name

    def rename(self, name):
        self._name = name

    @property
    def encrypted(self):
        """Returns whether this backup is encrypted. Encryption can
        be enabled at the agent level."""
        return self._encrypted

    @property
    def enabled(self):
        return self._enabled

    def connect(self, connection):
        self._connection = connection

    def _check_connection(self):
        if not self._connection:
            raise DisconnectedError()

    def _toggle(self, enabled=None):
        self._check_connection()
        url = '{0}/{1}/{2}/{3}'.format(self._connection.host,
                                       'backup-configuration',
                                       'enable', self.id)
        msg = json.dumps({'Enable': True if enabled else False})
        resp = self._connection.request(Http.post, url, data=msg)
        assert resp.json()['IsActive'] == enabled
        self._enabled = enabled

    def disable(self):
        self._toggle(False)

    def enable(self):
        self._toggle(True)

    @property
    def deleted(self):
        return self._deleted

    def delete(self):
        self._check_connection()
        url = '{0}/{1}/{2}'.format(self._connection.host,
                                   'backup-configuration',
                                   self.id)
        self._connection.request(Http.delete, url)
        self._deleted = True

    @property
    def schedule(self):
        return self._schedule

    def reschedule(self, schedule):
        self._schedule = schedule

    @property
    def inclusions(self):
        return self._inclusions

    def include(self, paths):
        """Updates the inclusions for this backup configuration.

        paths: a sequence of paths on the local filesystem.
        """
        self._set_paths(paths, are_exclusions=False)

    @property
    def exclusions(self):
        return self._exclusions

    def exclude(self, paths):
        """Updates the exclusions for this backup configuration.

        paths: a sequence of paths on the local filesystem.
        """
        self._set_paths(paths, are_exclusions=True)

    def _set_paths(self, paths, are_exclusions=False):
        data = {os.path.realpath(p) for p in paths}
        for path in data:
            _raise_if_not_exists(path)

        # prevent inconsistent state by checking inclusions
        # and exclusions don't contain common items
        if are_exclusions:
            _raise_if_overlapping(self._inclusions, data)
            self._exclusions.update(data)
        else:
            _raise_if_overlapping(self._exclusions, data)
            self._inclusions.update(data)

    def reload(self):
        """Captures the latest state from the API."""
        self._check_connection()
        url = '{0}/{1}/{2}'.format(self._connection.host,
                                   'backup-configuration',
                                   self.id)
        resp = self._connection.request(Http.get, url)
        parsed = resp.json()
        args = _args_from_dict(parsed)
        [setattr(self, k, v) for k, v in args.items()]

    def _create_or_update(self, creating=False):
        self._check_connection()
        url = None
        method = None

        if creating:
            url = '{0}/{1}'.format(self._connection.host,
                                   'backup-configuration')
            method = Http.post
        else:
            url = '{0}/{1}/{2}'.format(self._connection.host,
                                       'backup-configuration',
                                       self.id)
            method = Http.put

        data = to_json(self)
        resp = self._connection.request(method, url, data=data)
        return resp.json() if creating else None

    def create(self):
        """Takes the values stored locally and creates a new configuration.

        The configuration ID of this instance is updated to reflect the
        new configuration that was created.
        """
        resp = self._create_or_update(creating=True)
        self._config_id = resp['BackupConfigurationId']

    def update(self):
        """Takes the local values and updates the remote config."""
        return self._create_or_update(creating=False)



import rcbu.client.report as report
from rcbu.utils.bytes import dehumanize_bytes
import rcbu.common.duration as duration


def _args_from_dict(body):
    args = {
        '_agent_id': body['MachineAgentId'],
        '_machine_name': body['ComputerName'],
        '_restorable': body['CanRestore'],
        '_searched': {
            'files': int(body['FilesSearched']),
            'bytes': body['BytesSearched']
        },
        '_backup': {
            'files': int(body['FilesBackedUp']),
            'bytes': body['BytesBackedUp']
        }
    }
    args.update(report._args_from_dict(body))
    return args


def from_dict(backup_id, body):
    args = _args_from_dict(body)
    return BackupReport(backup_id, **args)


class BackupReport(report.Report):
    def __init__(self, report_id, **kwargs):
        report.Report.__init__(self, report_id, 'backup', **kwargs)

    def __repr__(self):
        form = ('<BackupReport id:{0} state:{1} ok:{2} '
                'started:{3} duration:{4} #errors:{5} bytes:{6}>')
        hours, minutes, seconds = duration.tuple(self.duration)
        pretty_duration = '{0}:{1:02}:{2:02}'.format(hours, minutes, seconds)
        return form.format(self.id, self.state, self.ok,
                           self.started.isoformat(), pretty_duration,
                           len(self.errors),
                           self.bytes_stored)

    @property
    def ok(self):
        return self._restorable

    @property
    def files_searched(self):
        return self._searched['files']

    @property
    def bytes_searched(self):
        return dehumanize_bytes(self._searched['bytes'])

    @property
    def files_stored(self):
        return self._backup['files']

    @property
    def bytes_stored(self):
        return dehumanize_bytes(self._backup['bytes'])



"""Utilities for handling byte quantities and strings."""


def dehumanize_bytes(human_bytes):
    """Convert a string in the format '2.23 GB' -> 2.23 * 10**30"""
    packed = human_bytes.split()
    amount, magnitude = packed[0], None
    if len(packed) == 2:
        magnitude = packed[1].upper()
    prefixes = ['YB', 'ZB', 'EB', 'PB', 'TB', 'GB', 'MB', 'KB']
    multipliers = [2**(i*10) for i in range(8, 0, -1)]
    magnitude_to_multiplier = {ma: mult for ma, mult in
                               zip(prefixes, multipliers)}
    multiplier = magnitude_to_multiplier.get(magnitude, 1)
    return int(float(amount) * multiplier)



import rcbu.common.status as status
import rcbu.common.activity as activity
from rcbu.common.http import Http


_predicates = {
    "backup_history": lambda j: j['Type'] == 'Backup' and not _is_running(j),
    "restore_history": lambda j: j['Type'] == 'Restore' and not _is_running(j),
    "cleanup_history": lambda j: j['Type'] == 'Cleanup' and not _is_running(j),
    "active_backups": lambda j: j['Type'] == 'Backup' and _is_running(j),
    "active_restores": lambda j: j['Type'] == 'Restore' and _is_running(j),
    "active_cleanups": lambda j: j['Type'] == 'Cleanup' and _is_running(j),
    "active": lambda j: _is_running(j),
    "not_active": lambda j: not _is_running(j)
}


def _is_running(job):
    return status.busy(job['CurrentState'])


def _jobs(connection, predicate, agent_id=None):
    url = ('{0}/activity'.format(connection.host) if agent_id is None else
           '{0}/system/activity/{1}'.format(connection.host, agent_id))
    resp = connection.request(Http.get, url, verify=False)
    resp.raise_for_status()
    return (activity.from_dict(b) for b in resp.json() if predicate(b))


def _any_running(connection, agent_id=None):
    try:
        next(_jobs(connection, _predicates['active'], agent_id))
    except StopIteration:
        return False
    return True


class ExposesActivities(object):
    def __init__(self, connection, oid=None):
        self._connection = connection
        self._id = oid

    def _listing(self, predicate):
        return _jobs(self._connection, predicate, self._id)

    @property
    def backup_history(self):
        return self._listing(_predicates['backup_history'])

    @property
    def restore_history(self):
        return self._listing(_predicates['restore_history'])

    @property
    def cleanup_history(self):
        return self._listing(_predicates['cleanup_history'])

    @property
    def active_backups(self):
        return self._listing(_predicates['active_backups'])

    @property
    def active_restores(self):
        return self._listing(_predicates['active_restores'])

    @property
    def active_cleanups(self):
        return self._listing(_predicates['active_cleanups'])

    @property
    def active(self):
        return self._listing(_predicates['active'])

    @property
    def history(self):
        return self._listing(_predicates['not_active'])

    @property
    def busy(self):
        return _any_running(self._connection, self._id)



import unittest

from httpretty import HTTPretty, httprettified
from requests.exceptions import HTTPError

import rcbu.common.auth as auth
from rcbu.common.constants import IDENTITY_TOKEN_URL


def _prepare_mock_post(status, body=''):
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=body)


class TestAuth(unittest.TestCase):

    @httprettified
    def test_auth_by_password_raises_if_invalid_creds(self):
        _prepare_mock_post(status=403)
        with self.assertRaises(HTTPError):
            auth.authenticate(password='bad', username='b')

    @httprettified
    def test_auth_by_apikey_raises_if_invalid_creds(self):
        _prepare_mock_post(status=403)
        with self.assertRaises(HTTPError):
            auth.authenticate(apikey='bad', username='b')

    @httprettified
    def test_auth_raises_if_neither_apikey_or_password_provided(self):
        _prepare_mock_post(status=200)
        with self.assertRaises(AssertionError):
            auth.authenticate(username='b')

    @httprettified
    def test_auth_returns_catalog_on_success(self):
        _prepare_mock_post(status=200, body='{"yay": 1}')
        catalog = auth.authenticate(username='b', password='good')
        self.assertEqual(catalog, {'yay': 1})

    @httprettified
    def test_get_token_by_password_raises_if_invalid_creds(self):
        _prepare_mock_post(status=403)
        with self.assertRaises(HTTPError):
            auth.get_token(password='bad', username='b')

    @httprettified
    def test_get_token_by_apikey_raises_if_invalid_creds(self):
        _prepare_mock_post(status=403)
        with self.assertRaises(HTTPError):
            auth.get_token(apikey='bad', username='b')

    @httprettified
    def test_get_token_returns_token_on_success(self):
        _prepare_mock_post(status=200,
                           body='{"access": {"token": {"id": "token"}}}')
        token = auth.get_token(username='b', password='good')
        self.assertEqual(token, 'token')



import datetime
import unittest

import rcbu.common.activity as activity
import tests.mock.activity as mock_activity


class TestActivity(unittest.TestCase):
    def setUp(self):
        self.activity = activity.from_dict(mock_activity.activity())

    def _matches_expected(self, prop, expected):
        value = getattr(self.activity, prop)
        self.assertEqual(value, expected)

    def test_id_matches_expected(self):
        self._matches_expected('id', 1)

    def test_type_matches_expected(self):
        self._matches_expected('type', 'Backup')

    def test_parent_matches_expected(self):
        self._matches_expected('parent', 2)

    def test_name_matches_expected(self):
        self._matches_expected('name', 'mock')

    def test_deleted_matches_expected(self):
        self._matches_expected('deleted', False)

    def test_source_matches_expected(self):
        self._matches_expected('source', 'mock 3')

    def test_destination_matches_expected(self):
        self._matches_expected('destination', None)
        self.activity._type = 'Restore'
        self._matches_expected('destination', 'mock 4')

    def test_state_matches_expected(self):
        self._matches_expected('state', 'Completed')

    def test_time_matches_expected(self):
        timestamp = int('/Date(1351118760000)/'[6:16])
        self._matches_expected('time',
                               datetime.datetime.fromtimestamp(timestamp))

    def test_repr_matches_expected(self):
        actual_form = repr(self.activity)
        expected_form = ('<BackupActivity id:1 name:mock state:Completed'
                         ' time:{0}>')
        self.assertEqual(actual_form,
                         expected_form.format(self.activity.time.isoformat()))



import json

from rcbu.client.command import Command
from rcbu.common.http import Http


def _args_from_dict(body):
    return {
        '_config_id': body['BackupConfigurationId'],
        '_state': body['CurrentState'],
        '_agent_id': body['MachineAgentId'],
        '_machine_name': body['MachineName'],
        '_key': {
            'modulus_hex': body['EncryptionKey']['ModulusHex'],
            'exponent_hex': body['EncryptionKey']['ExponentHex']
        }
    }


def from_dict(body, connection=None):
    return Backup(config_id=body['BackupConfigurationId'],
                  backup_id=body['BackupId'],
                  connection=connection,
                  **_args_from_dict(body))


class Backup(Command):
    def __init__(self, config_id, backup_id=None, connection=None, **kwargs):
        self._config_id = config_id
        Command.__init__(self, backup_id, 'backup', connection, **kwargs)

    def __repr__(self):
        form = ('<Backup id:{0} state:{1} running:{2}>')
        return form.format(self.id, self.state, self.running)

    def _action(self, starting):
        url = '{0}/backup/action-requested'.format(self._connection.host)
        action = 'StartManual' if starting else 'StopManual'
        action_id = self._config_id if starting else self.id
        data = {'Action': action, 'Id': action_id}
        resp = self._connection.request(Http.post, url,
                                        data=json.dumps(data))
        self._state = 'Preparing' if starting else 'Stopped'
        return resp

    def start(self):
        resp = self._action(starting=True)
        self._id = int(resp.json())
        return resp

    def stop(self):
        return self._action(starting=False)



class Weekdays(object):
    """A mock enum. When pep 435 rolls out, this can be upgraded to be a proper
    enum.

    ref: http://www.python.org/dev/peps/pep-0435/
    """
    Sunday = 0
    Monday = 1
    Tuesday = 2
    Wednesday = 3
    Thursday = 4
    Friday = 5
    Saturday = 6

    _to_string = {
        0: 'Sunday',
        1: 'Monday',
        2: 'Tuesday',
        3: 'Wednesday',
        4: 'Thursday',
        5: 'Friday',
        6: 'Saturday'
    }

    @classmethod
    def str(cls, value):
        return cls._to_string[value]



import datetime
import unittest

import pytz

from rcbu.client.connection import Connection
from tests.utils.credentials import Credentials


class TestConnect(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.creds = Credentials()
        cls.connection = Connection(cls.creds.name, apikey=cls.creds.key)

    def setup(self):
        self.connection = TestConnect.connection

    def test_token_makes_sense(self):
        self.assertEqual(len(self.connection.token), 32)

    def test_tenant_correct(self):
        self.assertEqual(self.connection.tenant, TestConnect.creds.tenant)

    def test_host_correct(self):
        self.assertEqual(self.connection.host,
                         'https://backup.api.rackspacecloud.com/v1.0')

    def test_api_version_correct(self):
        self.assertEqual(self.connection.api_version, '1.0')
        self.assertEqual(self.connection.api_version_tuple, (1, 0))

    def test_username_correct(self):
        self.assertEqual(self.connection.username, TestConnect.creds.name)

    def test_expires_makes_sense(self):
        self.assertGreater(self.connection.expires,
                           datetime.datetime.now(pytz.utc))



import unittest

import rcbu.common.duration as duration


class TestDuration(unittest.TestCase):
    def test_not_integer_seconds_throws_value_error(self):
        cases = ('10:10:aa', '10:aa:10', 'aa:10:10', '0:00:60', '0:60:00')
        for case in cases:
            with self.assertRaises(ValueError):
                duration.seconds(case)

    def test_seconds_works(self):
        cases = (  # string, seconds
            ('0:00:00', 0),
            ('0:00:01', 1),
            ('0:01:00', 60),
            ('1:00:00', 3600),
            ('23:59:59', 86399),
            ('1024:00:00', 3686400)
        )
        for case in cases:
            self.assertEqual(duration.seconds(case[0]), case[1])

    def test_tuple_works(self):
        cases = (  # value, hours, minutes, seconds
            (0, 0, 0, 0),
            (1, 0, 0, 1),
            (60, 0, 1, 0),
            (3600, 1, 0, 0),
            (86399, 23, 59, 59),
            (3686400, 1024, 0, 0)
        )
        for case in cases:
            self.assertEqual(duration.tuple(case[0]), tuple(case[1:]))



import unittest

import rcbu.utils.bytes as byte_utils


class TestBytes(unittest.TestCase):
    def _core(self, input_bytestring, expected):
        self.assertEqual(byte_utils.dehumanize_bytes(input_bytestring),
                         expected)

    def test_handles_unprefixed_string(self):
        self._core('1.1', 1)

    def test_handles_kilobytes(self):
        self._core('2 KB', 2048)

    def test_handles_megabytes(self):
        self._core('3.0 Mb', 3 * 2**20)

    def test_handles_zero_gigabytes(self):
        self._core('0 gB', 0)

    def test_handles_lowercased_terabytes(self):
        self._core('234 tb', 234 * 2**40)

    def test_handles_petabytes(self):
        self._core('322 pB', 322 * 2**50)

    def test_handles_exabytes(self):
        self._core('2.13 EB', 2.13 * 2**60)

    def test_handles_zettabytes(self):
        self._core('1 ZB', 2 ** 70)

    def test_handles_yottabytes(self):
        self._core('1 YB', 2 ** 80)



from random import randint

from rcbu.common.assertions import assert_bounded, assert_is_none
from rcbu.common.weekdays import Weekdays


class ScheduleFrequency(object):
    """A mock enum (PEP 435) for supported schedule frequencies."""
    Manual = 0
    Monthly = 1
    Weekly = 2
    Daily = 3
    Hourly = 4

    _to_api = {Manual: "Manually",
               Weekly: "Weekly",
               Daily: "Daily",
               Hourly: "Hourly",
               Monthly: "Monthly"}

    @classmethod
    def to_api(cls, value):
        return ScheduleFrequency._to_api.get(value, None)


def _validate_manual(interval, day_of_week, hour, minute):
    assert_is_none('Hourly interval', interval)
    assert_is_none('Day of week', day_of_week)
    assert_is_none('Hour', hour)
    assert_is_none('Minute', minute)


def _validate_weekly(interval, day_of_week, hour, minute):
    assert_is_none('Hourly interval', interval)
    assert_bounded('Day of week', 0, 6, day_of_week)
    assert_bounded('Hour', 0, 23, hour)
    assert_bounded('Minute', 0, 59, minute)


def _validate_daily(interval, day_of_week, hour, minute):
    assert_is_none('Hourly interval', interval)
    assert_is_none('Day of week', day_of_week)
    assert_bounded('Hour', 0, 23, hour)
    assert_bounded('Minute', 0, 59, minute)


def _validate_hourly(interval, day_of_week, hour, minute):
    assert_bounded('Hourly interval', 1, 23, interval)
    assert_is_none('Day of week', day_of_week)
    assert_is_none('Hour', hour)
    assert_is_none('Minute', minute)


# encode the validation process as a dictionary of valid schedule frequencies
_validate_fn = {
    ScheduleFrequency.Manual: _validate_manual,
    ScheduleFrequency.Weekly: _validate_weekly,
    ScheduleFrequency.Daily: _validate_daily,
    ScheduleFrequency.Hourly: _validate_hourly
}


class Schedule(object):
    """A class to simplify the management of RCBU backup scheduling."""
    def __init__(self, frequency, interval=None, day_of_week=None,
                 hour=None, minute=None):
        """
        args:
          frequency: Any value exposed by :see: ScheduleFrequency
          interval: For hourly backups - how many hours
                    between backups [0 - 23]
          day_of_week: Any value exposed by :see: rcbu.common.weekdays.Weekdays
          hour: For daily/weekly backups: the hour to
                schedule the backup [0 - 23]
          minute: For daily/weekly/hourly backups: the minute to
                  schedule the backup [0 - 59]
        raises:
          ValueError: for any argument out of range
        """
        self._validate(frequency, interval, day_of_week, hour, minute)
        self._frequency = frequency
        self._interval = interval
        self._day_of_week = day_of_week
        self._hour = hour
        self._minute = minute

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __repr__(self):
        form = ('<Schedule frequency:{0} weekday:{1} time:{2}')
        time = None
        if None not in (self.hour, self.minute):
            _form = '{0:02}:{1:02} {2}'
            time = _form.format(self.hour, self.minute, self.period)
        else:
            time = '*'
        weekday = self.weekday if self._day_of_week is not None else '*'
        form += (' every {0} hours'.format(self.interval) if self.interval else
                 '')
        form += '>'
        return form.format(self.frequency, weekday, time)

    @property
    def frequency(self):
        """Returns the frequency in a way that the API understand."""
        return ScheduleFrequency.to_api(self._frequency)

    @property
    def interval(self):
        """Returns the hourly interval used for this schedule."""
        return self._interval

    @property
    def weekday(self):
        """Returns the day of the week this schedule uses."""
        if self._day_of_week is None:
            return None
        return Weekdays.str(self._day_of_week)

    @property
    def hour(self):
        """Adjusts the hour to a 12-hour clock."""
        if self._hour is None:
            return None
        return self._hour if self._hour < 12 else self._hour - 12

    @property
    def minute(self):
        """Returns the minute this schedule uses."""
        return self._minute

    @property
    def period(self):
        """Returns 'AM' or 'PM', depending on the value of the hour."""
        if self._hour is None:
            return None
        return "AM" if self._hour < 12 else "PM"

    def _validate(self, frequency, interval, day_of_week, hour, minute):
        """Ensures that schedule args are valid, checking
        all the corner cases and all the boundaries."""
        assert_bounded('Frequency', 0, 4, frequency)
        _validate_fn[frequency](interval, day_of_week, hour, minute)

    def to_api(self):
        """Returns this schedule in a format that the API
        understands."""
        return {
            "Frequency": self.frequency,
            "StartTimeHour": self.hour,
            "StartTimeMinute": self.minute,
            "StartTimeAmPm": self.period,
            "DayOfWeekId": self._day_of_week,
            "HourInterval": self.interval
        }


def manually():
    """Returns a schedule appropriate for establishing a manual backup."""
    return Schedule(ScheduleFrequency.Manual, None, None, None, None)


def weekly(day_of_week,
           hour=None, minute=None):
    """Returns a schedule appropriate for establishing a weekly backup.

    args:
      day_of_week: On what day of the week should this
                   backup run? [Sunday - Saturday]
      hour: On what hour should this backup run? [0 - 23]
      minute: On what minute should this backup run? [0 - 59]
    """
    hour_ = randint(0, 23) if hour is None else hour
    minute_ = randint(0, 59) if minute is None else minute
    return Schedule(ScheduleFrequency.Weekly, None,
                    day_of_week=day_of_week, hour=hour_, minute=minute_)


def daily(hour=None, minute=None):
    """Returns a schedule appropriate for establishing a daily backup.

    args:
      hour: On what hour should this backup run? [0 - 23]
      minute: On what minute should this backup run? [0 - 59]
    """
    hour_ = randint(0, 23) if hour is None else hour
    minute_ = randint(0, 59) if minute is None else minute
    return Schedule(ScheduleFrequency.Daily, None,
                    day_of_week=None, hour=hour_, minute=minute_)


def hourly(interval):
    """Returns a schedule appropriate for establishing an hourly backup.

    args:
      interval: Hourly interval - every how many hours should
                this backup run? [0 - 23]
      minute: On what minute should this backup run? [0 - 59]
    """
    return Schedule(ScheduleFrequency.Hourly, interval=interval,
                    day_of_week=None, hour=None, minute=None)


def from_dict(resp):
    freq = resp['Frequency']
    minute = resp['StartTimeMinute']
    hour_ = resp['StartTimeHour']
    am_pm = resp['StartTimeAmPm']
    hour = hour_ + 12 if am_pm == 'PM' else hour_
    interval = resp['HourInterval']
    weekday_id = resp['DayOfWeekId']

    if freq == 'Manually':
        return manually()
    elif freq == 'Weekly':
        return weekly(weekday_id, hour, minute)
    elif freq == 'Daily':
        return daily(hour, minute)
    elif freq == 'Hourly':
        return hourly(interval)
    else:
        form = 'Invalid frequency {0}'.format(freq)
        raise ValueError(form)



import unittest

import rcbu.common.assertions as asserts


class TestAssertions(unittest.TestCase):
    def test_assert_is_none_throws_when_not_none(self):
        with self.assertRaises(ValueError):
            asserts.assert_is_none('test', 1)

    def test_assert_is_none_works(self):
        asserts.assert_is_none('test', None)

    def test_assert_bounded_throws_when_above_bounds(self):
        with self.assertRaises(ValueError):
            asserts.assert_bounded('test', 1, 3, 4)

    def test_assert_bounded_throws_when_below_bounds(self):
        with self.assertRaises(ValueError):
            asserts.assert_bounded('test', 1, 3, 0)

    def test_assert_bounded_works(self):
        [asserts.assert_bounded('test', 1, 3, i) for i in range(1, 4)]



MOCK_KEY = 'key'
MOCK_ENDPOINT = 'http://tacobackup.com/v1.0/912371'
MOCK_ENDPOINT_STRIPPED = 'http://tacobackup.com/v1.0'


def authenticate():
    backup_endpoint = [
        {
            'type': 'rax:backup',
            'endpoints': [
                {
                    'publicURL': MOCK_ENDPOINT
                }
            ]
        },
        {
            'type': 'something:else',
            'endpoints': [
                {
                    'publicURL': MOCK_ENDPOINT + '/not-right'
                }
            ]
        }
    ]

    reply = {
        'access': {
            'token': {
                'expires': '2013-06-05T05:20:28.306-05:00',
                'id': MOCK_KEY,
                'tenant': {
                    'id': 111111
                }
            },
            'serviceCatalog': backup_endpoint
        }
    }

    return reply



import json

from rcbu.common.activity_mixin import ExposesActivities
import rcbu.client.backup_configuration as backup_config
import rcbu.client.agent as agent
import rcbu.client.backup as backup
import rcbu.client.restore as restore
from rcbu.common.http import Http


class Client(ExposesActivities):
    def __init__(self, connection):
        self._connection = connection
        ExposesActivities.__init__(self, self._connection)

    def __repr__(self):
        return '<Client>'

    @property
    def agents(self):
        url = self._connection.host + '/user/agents'
        resp = self._connection.request(Http.get, url)
        return (agent.from_dict(a, connection=self._connection)
                for a in resp.json())

    @property
    def backup_configurations(self):
        url = self._connection.host + '/backup-configuration'
        resp = self._connection.request(Http.get, url)
        body = resp.json()
        return (backup_config.from_dict(config, self) for config in body)

    def get_agent(self, agent_id):
        url = '{0}/{1}/{2}'.format(self._connection.host, 'agent',
                                   agent_id)
        resp = self._connection.request(Http.get, url)
        return agent.from_dict(resp.json(), connection=self._connection)

    def get_backup_configuration(self, config_id):
        url = '{0}/{1}/{2}'.format(self._connection.host,
                                   'backup-configuration',
                                   config_id)
        resp = self._connection.request(Http.get, url)
        return backup_config.from_dict(resp.json())

    def get_backup_report(self, backup_id):
        url = '{0}/{1}/{2}/{3}'.format(self._connection.host, 'backup',
                                       'report',
                                       backup_id)
        resp = self._connection.request(Http.get, url)
        return resp.json()

    def create_backup(self, config):
        backup_action = backup.Backup(config.id, connection=self._connection)
        return backup_action

    def get_backup(self, backup_id):
        url = '{0}/{1}/{2}'.format(self._connection.host, 'backup', backup_id)
        resp = self._connection.request(Http.get, url)
        backup_action = backup.from_dict(resp.json(), self._connection)
        return backup_action

    def create_restore(self, backup_id, source_agent, destination_path,
                       destination_agent=None, overwrite=False):
        url = '{0}/{1}'.format(self._connection.host, 'restore')
        data = json.dumps({
            'BackupId': backup_id,
            'BackupMachineId': source_agent.id,
            'DestinationMachineId': (source_agent.id if not destination_agent
                                     else destination_agent.id),
            'DestinationPath': destination_path,
            'OverwriteFiles': overwrite
        })
        resp = self._connection.request(Http.put, url, data=data)
        restore_action = restore.from_dict(resp.json(),
                                           connection=self._connection)
        return restore_action

    def get_restore(self, restore_id):
        url = '{0}/restore/{1}'.format(self._connection.host, restore_id)
        resp = self._connection.request(Http.get, url)
        restore_action = restore.from_dict(resp.json(), self._connection)
        return restore_action



import binascii
import json
import os

from rcbu.common.http import Http
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5

from rcbu.common.activity_mixin import ExposesActivities
import rcbu.client.backup_configuration as backup_config
from rcbu.utils.bytes import dehumanize_bytes


def _args_from_dict(body):
    args = {
        '_version': body['AgentVersion'],
        '_vault_size': body['BackupVaultSize'],
        '_allow_cleanups': body['CleanupAllowed'],
        '_data_center': body['Datacenter'],
        '_ipv4': body['IPAddress'],
        '_machine_name': body['MachineName'],
        '_os': {
            'type': body['OperatingSystem'],
            'version': body['OperatingSystemVersion']
        },
        '_encrypted': body['IsEncrypted'],
        '_enabled': not body['IsDisabled'],
        '_online': body['Status']
    }
    return args


def from_dict(body, connection=None):
    args = _args_from_dict(body)
    return Agent(body.get('MachineAgentId', 0), connection, **args)


def from_file(path, connection=None):
    data = None
    with open(path, 'rt') as f:
        data = json.load(f)
    return from_dict(data, connection)


class Agent(ExposesActivities):
    def __init__(self, agent_id, connection=None, **kwargs):
        self._agent_id = agent_id
        self._connection = connection
        ExposesActivities.__init__(self, connection, agent_id)
        [setattr(self, k, v) for k, v in kwargs.items()]

    def __repr__(self):
        form = ('<Agent id:{0} name:{1} os:{2}, version:{3} '
                'data_center:{4} encrypted:{5} enabled:{6}>')
        return form.format(self.id, self.name, self.os, self.version,
                           self.data_center, self.encrypted, self.enabled)

    def connect(self, connection):
        self._connection = connection

    @property
    def id(self):
        return self._agent_id

    @property
    def version(self):
        return self._version

    @property
    def name(self):
        return self._machine_name

    @property
    def os(self):
        return '{0} {1}'.format(self._os['type'], self._os['version'])

    @property
    def data_center(self):
        return self._data_center

    def _fetch_property(self, name):
        url = '{0}/agent/{1}'.format(self._connection.host, self.id)
        resp = self._connection.request(Http.get, url)
        return resp.json()[name]

    @property
    def vault_size(self):
        if self._vault_size is None:
            self._vault_size = self._fetch_property('BackupVaultSize')
        return dehumanize_bytes(self._vault_size)

    @property
    def backup_configurations(self):
        url = '{0}/{1}/{2}/{3}'.format(self._connection.host,
                                       'backup-configuration', 'system',
                                       self.id)
        resp = self._connection.request(Http.get, url)
        return (backup_config.from_dict(b, self._connection)
                for b in resp.json())

    @property
    def online(self):
        '''Lazily-loaded property. Queries the API if the status is unknown.
        Otherwise, it returns the last known state.'''
        if self._online == 'Unknown':
            self._online = self._fetch_property('Status')
        return self._online == 'Online'

    @property
    def encrypted(self):
        return self._encrypted

    def _public_key_path(self):
        name = 'public-key.pem'
        if os.name != 'nt':
            return os.path.join('/etc/driveclient', name)
        else:
            return os.path.join(os.path.expandvars('%appdata%'),
                                'Local', 'Driveclient', name)

    def encrypt(self, password):
        public_key_path = self._public_key_path()
        public_key = None
        with open(public_key_path) as f:
            public_key = f.read()

        pkey = RSA.importKey(public_key)
        cipher = PKCS1_v1_5.new(pkey)
        encrypted_password = cipher.encrypt(password)
        hex_pass = binascii.hexlify(encrypted_password).decode()
        url = '{0}/{1}/{2}'.format(self._connection.host, 'agent', 'encrypt')
        data = json.dumps({
            'MachineAgentId': self.id,
            'EncryptedPasswordHex': hex_pass,
        })
        self._connection.request(Http.post, url, data=data)
        self._encrypted = True

    @property
    def enabled(self):
        return self._enabled

    def _toggle(self, enabled=True):
        url = '{0}/{1}/{2}'.format(self._connection.host, 'agent', 'enable')
        data = json.dumps({
            'MachineAgentId': self.id,
            'Enable': enabled
        })
        self._connection.request(Http.post, url, data=data)
        self._enabled = enabled

    def enable(self):
        self._toggle(enabled=True)

    def disable(self):
        self._toggle(enabled=False)

    def delete(self):
        url = '{0}/{1}/{2}'.format(self._connection.host, 'agent', 'delete')
        data = json.dumps({'MachineAgentId': self.id})
        self._connection.request(Http.post, url, data=data)



"""A module that constructs mock report dictionaries."""


def report(config_id=1, name='mock', state='Completed',
           start='/Date(1351118760000)/',
           end='/Date(1351118760001)/',
           duration='00:00:00', errors=None,
           outcome='OK', diagnostics='OK'):
    """Base mock report."""
    return {
        'BackupConfigurationId': config_id,
        'BackupConfigurationName': name,
        'State': state,
        'StartTime': start,
        'CompletedTime': end,
        'Duration': duration,
        'NumErrors': len(errors) if errors else 0,
        'ErrorList': errors,
        'Reason': outcome,
        'Diagnostics': diagnostics
    }


def backup_report(agent_id=1, machine_name='mock', restorable=True,
                  files_searched=0, bytes_searched='2 GB',
                  files_stored=0, bytes_stored='2 GB',
                  **kwargs):
    """Returns a mock backup report."""
    base = report(**kwargs)
    base.update({
        'MachineAgentId': agent_id,
        'ComputerName': machine_name,
        'CanRestore': restorable,
        'FilesSearched': files_searched,
        'BytesSearched': bytes_searched,
        'FilesBackedUp': files_stored,
        'BytesBackedUp': bytes_stored
    })
    return base


def restore_report(backup_id=1, files_restored=0, bytes_restored='2 GB',
                   destination=1, path='/mock', **kwargs):
    """Returns a mock restore report."""
    base = report(**kwargs)
    base.update({
        'BackupConfigurationId': backup_id,
        'NumFilesRestored': files_restored,
        'NumBytesRestored': bytes_restored,
        'RestoreDestinationMachineId': destination,
        'RestoreDestination': path
    })
    return base



import datetime
import re


DATE_MATCHER = re.compile(r'/Date\((\d{13})\)/')


def parse(date):
    """Given a .Net Date object, returns a Python datetime object."""
    # .Net Date objects differ in numeric format from Python datetime
    # objects. We must parse out the milliseconds instead of passing
    # the value directly to datetime, or we get a:
    # ValueError: year is out of range
    stamp = DATE_MATCHER.match(date).group(1)
    precise_timestamp = '{0}.{1}'.format(stamp[:10], stamp[10:])
    return datetime.datetime.fromtimestamp(float(precise_timestamp))



import json
import os


def _parse_config():
    path = os.path.join(os.path.expanduser('~'),
                        '.pysdk', 'credentials.json')
    data = None
    try:
        with open(path) as f:
            data = json.load(f)
    except IOError:
        form = 'Did you remember to install {0}?'
        raise RuntimeError(form.format(path))

    return (data.get('username', None),
            data.get('apikey', None),
            data.get('email', None),
            data.get('tenant', None))


class Credentials(object):
    def __init__(self):
        name, key, email, tenant = _parse_config()
        self._name = name
        self._key = key
        self._email = email
        self._tenant = tenant

    @property
    def name(self):
        return self._name

    @property
    def key(self):
        return self._key

    @property
    def email(self):
        return self._email

    @property
    def tenant(self):
        return self._tenant



import rcbu.common.exceptions as exceptions
import rcbu.common.duration as duration
import rcbu.utils.date as date


def _args_from_dict(body):
    return {
        '_config_id': body['BackupConfigurationId'],
        '_config_name': body['BackupConfigurationName'],
        '_state': body['State'],
        '_time': {
            'start': body['StartTime'],
            'end': body['CompletedTime'],
            'duration': body['Duration']
        },
        '_num_errors': body['NumErrors'],
        '_outcome': body['Reason'],
        '_diagnostics': body['Diagnostics'],
        '_errors': body['ErrorList']
    }


_error_class = {
    'backup': exceptions.BackupFailed,
    'restore': exceptions.RestoreFailed
}


class Report(object):
    def __init__(self, report_id, report_type, **kwargs):
        self.report_id = report_id
        self._type = report_type
        [setattr(self, k, v) for k, v in kwargs.items()]

    @property
    def id(self):
        return self.report_id

    @property
    def state(self):
        return self._state

    @property
    def errors(self):
        return self._errors

    @property
    def outcome(self):
        return self._outcome

    @property
    def ok(self):
        return len(self.errors) == 0

    @property
    def diagnostics(self):
        return self._diagnostics

    @property
    def started(self):
        return date.parse(self._time['start'])

    @property
    def ended(self):
        return date.parse(self._time['end'])

    @property
    def duration(self):
        return duration.seconds(self._time['duration'])

    def raise_if_not_ok(self):
        if not self.ok:
            raise _error_class[self._type](self)





def activity(activity_id=1, type_tag='Backup', parent=2, name='mock',
             deleted=False, source_id=3, source_name='mock',
             destination_id=4, destination_name='mock',
             state='Completed', time='/Date(1351118760000)/'):
    return {
        'ID': activity_id,
        'Type': type_tag,
        'ParentId': parent,
        'DisplayName': name,
        'IsBackupConfigurationDeleted': deleted,
        'SourceMachineAgentId': source_id,
        'SourceMachineName': source_name,
        'DestinationMachineAgentId': destination_id,
        'DestinationMachineName': destination_name,
        'CurrentState': state,
        'TimeOfActivity': time
    }



import json
import unittest
import sys

from httpretty import HTTPretty, httprettified
import mock

from rcbu.client.connection import Connection
from rcbu.client.client import Client
from rcbu.common.auth import IDENTITY_TOKEN_URL
import tests.mock.auth as mock_auth
import tests.mock.agent as agent_mock
import tests.mock.rsa as mock_rsa
import tests.mock.configuration as mock_config
import rcbu.client.agent as agent


def _mock_auth(status):
    reply = mock_auth.authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


class TestLoadAgent(unittest.TestCase):
    '''The mock framework (in the stdlib in Python 3!) can do some
    pretty magical things. Below, we're avoiding the disk entirely
    by mocking the builtin open method and having it return our
    mock agent JSON and asserting that we parse it correctly.'''
    def test_can_parse_agent_from_json_file(self):
        m = mock.mock_open(read_data=json.dumps(agent_mock.agent()))
        open_fn_name = ('builtins.open' if sys.version_info[0] == 3 else
                        '__builtin__.open')
        with mock.patch(open_fn_name, m, create=False):
            conn_mock = mock.MagicMock()
            conn_mock.host = 'a'
            conn_mock.token = 'a'
            ag = agent.from_file('test', conn_mock)
            self.assertEqual(ag.name, 'mock')
            self.assertEqual(ag.id, '1')
            self.assertEqual(ag.os, 'linux 13.04')


class TestAgent(unittest.TestCase):
    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.conn = Connection('1', '2')
        self.client = Client(self.conn)
        self.agent = agent.from_dict(agent_mock.agent(), self.conn)

    def test_name_matches_expected(self):
        self.assertEqual(self.agent.name, 'mock')

    def test_version_matches_expected(self):
        self.assertEqual(self.agent.version, '1.065959')

    def test_id_matches_expected(self):
        self.assertEqual(self.agent.id, '1')

    def test_os_matches_expected(self):
        self.assertEqual(self.agent.os, 'linux 13.04')

    def test_dc_matches_expected(self):
        self.assertEqual(self.agent.data_center, 'ORD')

    def test_encrypted_matches_expected(self):
        self.assertEqual(self.agent.encrypted, False)

    def test_enabled_matches_expected(self):
        self.assertEqual(self.agent.enabled, True)

    def test_vault_size_matches_expected(self):
        self.assertEqual(self.agent.vault_size, 2 * 2 ** 30)

    @httprettified
    def test_online_fetches_from_API_if_status_unknown(self):
        url = '{0}/agent/{1}'.format(self.conn.host, self.agent.id)
        HTTPretty.register_uri(HTTPretty.GET, url, status=200,
                               body=json.dumps({'Status': 'Online'}))
        self.assertEqual(self.agent.online, True)

    @httprettified
    def test_vault_size_fetches_from_API_if_none(self):
        url = '{0}/agent/{1}'.format(self.conn.host, self.agent.id)
        HTTPretty.register_uri(HTTPretty.GET, url, status=200,
                               body=json.dumps({'BackupVaultSize': '2 GB'}))
        self.assertEqual(self.agent.vault_size, 2 * 2**30)

    def test_online_returns_correct_result(self):
        self.agent._online = 'Online'
        self.assertTrue(self.agent.online)
        self.agent._online = 'Offline'
        self.assertFalse(self.agent.online)

    @httprettified
    def test_fetch_backup_configurations_works(self):
        url = '{0}/backup-configuration/system/{1}'.format(self.conn.host,
                                                           self.agent.id)
        data = json.dumps([mock_config.backup_configuration()
                           for i in range(10)])
        HTTPretty.register_uri(HTTPretty.GET, url, status=200, body=data)
        confs = list(self.agent.backup_configurations)
        self.assertEqual(len(confs), 10)
        for conf in confs:
            self.assertEqual(conf.name, 'mock')

    @httprettified
    def test_encrypt_works(self):
        url = '{0}/agent/encrypt'.format(self.conn.host)
        HTTPretty.register_uri(HTTPretty.POST,
                               url, status=204)
        self.assertEqual(self.agent.encrypted, False)

        # This is some vicious mocking. In order:
        # 1. Mock the built-in open so we read in a mock PEM
        # 2. Mock PyCrypto's importKey so we avoid reading from /dev/urandom
        # 3. Mock PyCrypto's Cipher generator so we avoid randgen
        # 4. Mock PyCrypto's Cipher.encrypt since its already a mock
        # 5. Ensure the call can complete and the agent is encrypted
        m = mock.mock_open(read_data=mock_rsa.public_key())
        open_fn_name = ('builtins.open' if sys.version_info[0] == 3 else
                        '__builtin__.open')
        with mock.patch(open_fn_name, m, create=True):
            with mock.patch('Crypto.PublicKey.RSA.importKey') as key:
                key.return_value = 'woot'
                with mock.patch('Crypto.Cipher.PKCS1_v1_5.new') as cipher:
                    cipher.return_value = mock.MagicMock()
                    cipher.return_value.encrypt.return_value = b'awesome'
                    self.agent.encrypt('sweet_tacos')

        self.assertEqual(self.agent.encrypted, True)

    @httprettified
    def _toggle_test(self, enabled=True):
        HTTPretty.register_uri(HTTPretty.POST,
                               '{0}/agent/enable'.format(self.conn.host),
                               status=200)
        if enabled:
            self.agent.enable()
        else:
            self.agent.disable()
        self.assertEqual(self.agent.enabled, enabled)

    def test_enabling_agent_works(self):
        self._toggle_test(enabled=True)

    def test_disabling_agent_works(self):
        self._toggle_test(enabled=False)

    def test_connect_works(self):
        self.agent.connect(None)
        self.assertEqual(self.agent._connection, None)
        self.agent.connect(self.conn)
        self.assertIs(self.agent._connection, self.conn)

    @httprettified
    def test_delete_works(self):
        '''For this test, it's enough to verify that delete can be
        called without throwing any errors. This also checks that the URL
        that is called if of the expected format. If the URL fails to match,
        then it bypasses the HTTPretty mock and throws a 403.'''
        HTTPretty.register_uri(HTTPretty.POST,
                               '{0}/agent/delete'.format(self.conn.host),
                               status=204)
        self.agent.delete()



import time


class Timer():
    """Context Manager class for timing blocks of code"""
    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, *args):
        self.end = time.time()
        self.elapsed = self.end - self.start



import unittest

import rcbu.client.restore_report as restore_report
import tests.mock.report as mock_report
from rcbu.common.exceptions import RestoreFailed


class TestRestoreReport(unittest.TestCase):
    def setUp(self):
        mock = mock_report.restore_report(errors=['explosions'])
        self.report = restore_report.from_dict(1, mock)

    def test_repr_matches_expected(self):
        form = ('<RestoreReport id:{0} state:{1} ok:{2} started:{3} '
                'duration:{4} #errors:{5} bytes:{6}>')
        self.assertEqual(repr(self.report),
                         form.format(self.report.id, self.report.state,
                                     self.report.ok,
                                     self.report.started.isoformat(),
                                     '0:00:00', len(self.report.errors),
                                     self.report.bytes_restored))

    def test_id_matches_expected(self):
        self.assertEqual(self.report.id, 1)

    def test_errors_match_expected(self):
        self.assertEqual(len(self.report.errors), 1)
        self.assertEqual(self.report.errors[0], 'explosions')

    def test_ok_matches_expected(self):
        self.assertEqual(self.report.ok, False)

    def test_raises_if_not_restorable(self):
        with self.assertRaises(RestoreFailed):
            self.report.raise_if_not_ok()

    def test_restored_matches_expected(self):
        self.assertEqual(self.report.files_restored, 0)
        self.assertEqual(self.report.bytes_restored, 2 * 2**30)

    def test_destination_matches_expected(self):
        self.assertEqual(self.report.destination, '/mock 1')



import unittest

import mock

from rcbu.utils.perf import Timer


class TestTimer(unittest.TestCase):
    def test_elapsed_time_is_correct(self):
        with mock.patch('time.time') as time_mock:
            time_mock.return_value = 10
            with Timer() as t:
                time_mock.return_value = 11
            self.assertEqual(t.elapsed, 1)



import os
import re
from setuptools import setup, find_packages

pip_requires = os.path.join(os.getcwd(), 'requirements.txt')
test_requires = os.path.join(os.getcwd(), 'test-requirements.txt')

def credentials():
    install_path = os.path.join(os.path.expanduser('~'),
                                '.pysdk')
    if os.path.exists(os.path.join(install_path, 'credentials.json')):
        return ('.', ['setup.py'])  # null install
    conf_path = os.path.join('conf', 'credentials.json')
    return (install_path, [conf_path])


def file_lines(path):
    reqs = None
    with open(path, 'rt') as f:
        reqs = f.read().split()
    return reqs


def parse_version():
    data = None
    with open('README.rst', 'rt') as f:
        data = f.read()
    return '.'.join(re.search(r':version: (\d+)\.(\d+)\.(\d+)', data).groups())


data_files = []
data_files.append(credentials())
setup(
    name='rackspace-backup-client',
    version=parse_version(),
    author='Alejandro Cabrera',
    license='Apache License (2.0)',
    keywords='rackspace, Cloud Backup, client',
    author_email='alejandro.cabrera@rackspace.com',
    description='A Python client for the Rackspace Cloud Backup API.',
    long_description=open('README.rst').read(),
    url='https://github.com/rackerlabs/python-cloudbackup-sdk',
    packages=find_packages(),
    zip_safe=False,
    install_requires=file_lines(pip_requires),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
    data_files=data_files
)



import rcbu.client.report as report
from rcbu.utils.bytes import dehumanize_bytes
import rcbu.common.duration as duration


def _args_from_dict(body):
    args = {
        '_backup_id': body['BackupConfigurationId'],
        '_restored': {
            'files': int(body['NumFilesRestored']),
            'bytes': body['NumBytesRestored']
        },
        '_destination': {
            'id': body['RestoreDestinationMachineId'],
            'path': body['RestoreDestination']
        }
    }
    args.update(report._args_from_dict(body))
    return args


def from_dict(restore_id, body):
    args = _args_from_dict(body)
    return RestoreReport(restore_id, **args)


class RestoreReport(report.Report):
    def __init__(self, report_id, **kwargs):
        report.Report.__init__(self, report_id, 'restore', **kwargs)

    def __repr__(self):
        form = ('<RestoreReport id:{0} state:{1} ok:{2} started:{3} '
                'duration:{4} #errors:{5} bytes:{6}>')
        hours, minutes, seconds = duration.tuple(self.duration)
        return form.format(self.id, self.state, self.ok,
                           self.started.isoformat(),
                           '{0}:{1:02}:{2:02}'.format(hours, minutes, seconds),
                           len(self.errors), self.bytes_restored)

    @property
    def files_restored(self):
        return self._restored['files']

    @property
    def bytes_restored(self):
        return dehumanize_bytes(self._restored['bytes'])

    @property
    def destination(self):
        """Returns a string in the form: 'path id'"""
        return '{0} {1}'.format(self._destination['path'],
                                self._destination['id'])



from rcbu.common import auth
from rcbu.common import exceptions
from rcbu.common import http

from dateutil import parser
import requests
import six


def _normalize_endpoint(url):
    idx = url.rfind('/')
    return url[:idx]


def _find_given_region(target, region):
    region_matcher = lambda x: x['region'] == region.upper()
    try:
        return next(six.moves.filter(region_matcher,
                                     target['endpoints']))
    except StopIteration:
        return None


def _find_generic_endpoint(target):
    regionless_matcher = lambda x: u'region' not in x
    try:
        return next(six.moves.filter(regionless_matcher,
                                     target['endpoints']))
    except StopIteration:
        return None


def _find_backup_endpoint(catalogue, region):
    backup_matcher = lambda x: x['type'] == u'rax:backup'
    target = next(six.moves.filter(backup_matcher, catalogue))

    # NOTE(cabrera): attempt to grab a region-specific endpoint. If
    # this fails, then either the region provided was not valid or
    # there are no region-specific endpoints for this tenant.
    endpoint = None
    if region:
        endpoint = _find_given_region(target, region)

    # NOTE(cabrera): if no endpoint was detected for a specific region
    # or no region was provided, attempt to grab a generic endpoint
    if (not region) or (not endpoint):
        endpoint = _find_generic_endpoint(target)

    return _normalize_endpoint(endpoint[u'publicURL'])


class Connection(object):
    def __init__(self, username, apikey=None, password=None, region=None):
        assert apikey or password

        resp = None
        # If apikey and password are given, give priority to
        # authentication via API key.
        if apikey:
            resp = auth.authenticate(username, apikey=apikey)
        else:
            resp = auth.authenticate(username, password=password)

        self._token = resp['access']['token']['id']
        endpoints = resp['access']['serviceCatalog']

        try:
            self._endpoint = _find_backup_endpoint(endpoints, region)
        except TypeError:
            raise exceptions.NoEndpointFound(username, region)

        self._username = username
        self._tenant = resp['access']['token']['tenant']['id']
        self._expiry = resp['access']['token']['expires']
        self._session = requests

    def __repr__(self):
        msg = ('<Connection host:{0} tenant:{1} username:{2} expires:{3}>')
        return msg.format(self.host, self.tenant,
                          self.username,
                          self.expires.isoformat())

    @property
    def token(self):
        return self._token

    @property
    def tenant(self):
        return self._tenant

    @property
    def username(self):
        return self._username

    @property
    def host(self):
        return self._endpoint

    @property
    def api_version(self):
        return self.host.split('/')[-1].lstrip('v')

    @property
    def api_version_tuple(self):
        return tuple(int(i) for i in self.api_version.split('.'))

    @property
    def expires(self):
        date = parser.parse(self._expiry)
        return date

    def request(self, method, url, headers=None, data=None, verify=True):
        '''Method is an http.Http enum, such as http.Http.get.'''
        # todo: add reauth when token is nearing expiration here
        headers_ = {
            'x-auth-token': self.token,
            'content-type': 'application/json',
            'user-agent': 'python-cloudbackup-sdk'
        }
        if headers:
            headers_.update(headers)
        call = getattr(self._session, http.enum_to_method(method))
        resp = call(url, headers=headers_, data=data, verify=verify)
        resp.raise_for_status()
        return resp



from rcbu.common.http import Http


int_to_status = lambda n: {
    0: 'Creating',
    1: 'Queued',
    2: 'InProgress',
    3: 'Completed',
    4: 'Stopped',
    5: 'Failed',
    6: 'StartRequested',
    7: 'StopRequested',
    8: 'CompletedWithErrors',
    9: 'Preparing'
}[n]

BUSY_STATUS = ('StartRequested', 'Creating', 'InProgress',
               'StopRequested', 'Queued', 'Preparing')
DONE_STATUS = ('Completed', 'CompletedWithErrors', 'Stopped',
               'Skipped', 'Failed', 'Missed')


def busy(status):
    return status in BUSY_STATUS


_state_interpret = {
    'restore': lambda r: int_to_status(r.json()['RestoreStateId']),
    'backup': lambda r: r.json()['CurrentState']
}


class Status(object):
    def __init__(self, command_id, command_type, connection):
        self._id = command_id
        self._type = command_type
        self._connection = connection

    @property
    def id(self):
        return self._id

    @property
    def state(self):
        url = '{0}/{1}/{2}'.format(self._connection.host,
                                   self._type, self.id)
        resp = self._connection.request(Http.get, url)
        return _state_interpret[self._type](resp)



import unittest

from rcbu.client.connection import Connection
from rcbu.client.client import Client
from tests.utils.credentials import Credentials


class TestClientBasic(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        creds = Credentials()
        cls.connection = Connection(creds.name,
                                    apikey=creds.key)
        cls.client = Client(cls.connection)

    def setUp(self):
        self.client = TestClientBasic.client

    def test_has_agents(self):
        agents = list(self.client.agents)
        self.assertGreater(len(agents), 0)

    def test_get_agent_by_id(self):
        agent = list(self.client.agents)[-1]
        agent2 = self.client.get_agent(agent.id)
        self.assertEqual(agent.id, agent2.id)

    def test_client_can_fetch_backup_history(self):
        backups = list(self.client.backup_history)
        self.assertGreaterEqual(len(backups), 0)

    def test_client_can_fetch_restore_history(self):
        restores = list(self.client.restore_history)
        self.assertGreaterEqual(len(restores), 0)

    def test_client_can_fetch_active_backups(self):
        backups = list(self.client.active_backups)
        self.assertGreaterEqual(len(backups), 0)
        for backup in backups:
            self.assertEqual(backup.type, 'Backup')

    def test_client_can_fetch_active_restores(self):
        restores = list(self.client.active_restores)
        self.assertGreaterEqual(len(restores), 0)
        for restore in restores:
            self.assertEqual(restore.type, 'Restore')

    def test_client_busy_returns_true_or_false(self):
        self.assertIn(self.client.busy, [True, False])



import time

from rcbu.utils.perf import Timer
import rcbu.client.backup_report as backup_report
import rcbu.client.restore_report as restore_report
import rcbu.common.status as status
from rcbu.common.http import Http


class Command(object):
    def __init__(self, command_id, command_type, connection, **kwargs):
        self._id = command_id
        self._type = command_type
        self._connection = connection
        [setattr(self, k, v) for k, v in kwargs.items()]

    @property
    def running(self):
        return self.state in status.BUSY_STATUS

    def _fetch_state(self, reload=False):
        if reload:
            self._state = status.Status(self.id, self._type,
                                        self._connection).state
        return self.state

    @property
    def state(self):
        return getattr(self, '_state', 'Unknown')

    @property
    def id(self):
        return self._id

    def connect(self, connection):
        self._connection = connection

    def _report(self, body):
        return {
            'backup': lambda body: backup_report.from_dict(self.id, body),
            'restore': lambda body: restore_report.from_dict(self.id, body)
        }[self._type](body)

    @property
    def report(self):
        url = '{0}/{1}/report/{2}'.format(self._connection.host,
                                          self._type, self.id)
        resp = self._connection.request(Http.get, url)
        return self._report(resp.json())

    def _is_done(self):
        state = self._fetch_state(reload=True)
        return state in status.DONE_STATUS

    def wait_for_completion(self, poll_interval=60, timeout=None):
        time_waited = 0
        while not self._is_done():
            with Timer() as start:
                time.sleep(poll_interval)
            time_waited += start.elapsed
            if timeout and time_waited > timeout:
                raise RuntimeError('{} took too long.'.format(self._type))



IDENTITY_TOKEN_URL = 'https://identity.api.rackspacecloud.com/v2.0/tokens'
ENCRYPT_KEY_URL = ('http://www.rackspace.com/knowledge_center/'
                   'article/generating-your-encrypted-key-in-cloud-backup')



import json
import unittest

from httpretty import HTTPretty, httprettified
from requests.exceptions import HTTPError
from dateutil import parser

from rcbu.client.connection import Connection
from rcbu.client.client import Client
from rcbu.common.constants import IDENTITY_TOKEN_URL
from tests.mock.auth import (
    authenticate, MOCK_ENDPOINT_STRIPPED, MOCK_KEY
)


def _mock_auth(status):
    reply = authenticate()
    HTTPretty.register_uri(HTTPretty.POST, IDENTITY_TOKEN_URL,
                           status=status, body=json.dumps(reply))


class TestValidConnection(unittest.TestCase):

    @httprettified
    def setUp(self):
        _mock_auth(200)
        self.conn = Connection('a', password='a')
        self.client = Client(self.conn)

    def test_connection_has_correct_properties(self):
        self.assertEqual(self.conn.token, MOCK_KEY)
        self.assertEqual(self.conn.host, MOCK_ENDPOINT_STRIPPED)

    @httprettified
    def test_agents_raises_403_on_invalid_auth(self):
        url = self.conn.host + '/user/agents'
        HTTPretty.register_uri(HTTPretty.GET, url, status=403)
        with self.assertRaises(HTTPError):
            self.client.agents

    @httprettified
    def test_backup_configurations_raises_403_on_invalid_auth(self):
        url = self.conn.host + '/backup-configuration'
        HTTPretty.register_uri(HTTPretty.GET, url, status=403)
        with self.assertRaises(HTTPError):
            self.client.backup_configurations

    def test_repr_matches_expected(self):
        form = ('<Connection host:{0} tenant:{1} username:{2} expires:{3}>')
        self.assertEqual(repr(self.conn),
                         form.format(self.conn.host, self.conn.tenant,
                                     self.conn.username,
                                     self.conn.expires.isoformat()))

    def test_host(self):
        self.assertEqual(self.conn.host, MOCK_ENDPOINT_STRIPPED)

    def test_version(self):
        self.assertEqual(self.conn.api_version, '1.0')

    def test_version_tuple(self):
        self.assertEqual(self.conn.api_version_tuple, (1, 0))

    def test_tenant_matches_expected(self):
        self.assertEqual(self.conn.tenant, 111111)

    def test_expires_matches_expected(self):
        self.assertEqual(self.conn.expires,
                         parser.parse(self.conn._expiry))
